import utils
from models.area_email import AreaEmail
from models.collect_data import CollectData
from models.process_data import ProcessData
from models.process_result_data import ProcessResultData
from stores.data_store_interface import IDataStore


class MysqlDataStore(IDataStore):

    _collectData = CollectData()
    _processData = ProcessData()
    _processResultData = ProcessResultData()
    _areaEmail = AreaEmail()

    def __init__(self):
        self._collect_size = utils.get_config_int("save.collect_batch_size", 1)
        self._collect_list = []
        self._process_size = utils.get_config_int("save.process_batch_size", 1)
        self._process_list = []
        self._process_result_list = []

    def query_one_collect_url(self, url: str) -> str | None:
        return self._collectData.fetch_one_url(url)

    def insert_collect_data(self, data: CollectData, is_batch=True):
        if not is_batch:
            self._collectData.insert(data)
            utils.get_logger().info(f"保存 采集数据 到数据库: {data.url}")
        else:
            self._collect_list.append(data)
            self.save_collect_data()

    def save_collect_data(self, is_force=False):
        if (is_force and len(self._collect_list) > 0) or len(
            self._collect_list
        ) >= self._collect_size:
            utils.get_logger().info(
                "批量保存 采集数据 到数据库，数量: " + str(len(self._collect_list))
            )
            self._collectData.insert_batch(self._collect_list)
            self._collect_list = []

    def query_urls_to_process(self):
        return self._collectData.fetch_urls_to_process()

    def query_one_collect_by_url(self, url):
        return self._collectData.fetch_one_collect_by_url(url)

    def query_one_process_by_url(self, url):
        return self._processData.fetch_one_process_by_url(url)

    def query_one_process_by_no(self, no):
        return self._processData.fetch_one_process_by_no(no)

    def insert_process_data(self, data: ProcessData, is_batch=True):
        if not is_batch:
            self._processData.insert(data)
            self._collectData.set_process(data.url)
            utils.get_logger().info(f"保存 处理数据 到数据库: {data.url}")
        else:
            self._process_list.append(data)
            self.save_process_data()

    # 插入到数据库时会把CollectData设为已处理
    def save_process_data(self, is_force=False):
        if (is_force and len(self._process_list) > 0) or len(
            self._process_list
        ) >= self._process_size:
            utils.get_logger().info(
                f"批量保存 处理数据 到数据库，数量: {str(len(self._process_list))}"
            )
            self._processData.insert_batch(self._process_list)
            urls = [item.url for item in self._process_list]
            self._collectData.set_process_list(urls)
            self._process_list = []

    def set_collect_process(self, url):
        return self._collectData.set_process(url)

    def set_process_other_urls(self, url, other_urls: str):
        return self._processData.set_other_urls(url, other_urls)

    def query_one_process_result_by_url(self, url):
        return self._processResultData.fetch_one_process_by_url(url)

    def query_one_process_result_by_no(self, no):
        return self._processResultData.fetch_one_process_by_no(no)

    def insert_process_result_data(self, data: ProcessResultData, is_batch=True):
        if not is_batch:
            self._processResultData.insert(data)
            self._collectData.set_process(data.url)
            utils.get_logger().info(f"保存 处理数据结果 到数据库: {data.url}")
        else:
            self._process_result_list.append(data)
            self.save_process_result_data()

    def save_process_result_data(self, is_force=False):
        if (is_force and len(self._process_result_list) > 0) or len(
            self._process_result_list
        ) >= self._process_size:
            utils.get_logger().info(
                f"批量保存 处理数据结果 到数据库，数量: {str(len(self._process_result_list))}"
            )
            self._processResultData.insert_batch(self._process_result_list)
            urls = [item.url for item in self._process_result_list]
            self._collectData.set_process_list(urls)
            self._process_result_list = []

    def set_process_result_other_urls(self, url, other_urls: str):
        return self._processResultData.set_other_urls(url, other_urls)

    def query_to_send(self):
        return self._processData.fetch_not_send()

    def query_to_report_by_date(self, start_date, end_date):
        return self._processResultData.fetch_to_report_by_date(start_date, end_date)

    def set_send(self, no: str):
        self._processData.set_send(no)

    def query_all_emails(self) -> list[AreaEmail]:
        return self._areaEmail.fetch_all()

    def query_all_virtual_emails(self) -> list[AreaEmail]:
        return self._areaEmail.fetch_all_virtual()

    def get_email_by_area(self, area: str) -> str:
        return self._areaEmail.fetch_one_by_area(area)

    def query_master_email(self) -> str:
        return self._areaEmail.fetch_master_email()

    def update_area_email_area_by_name(self, name: str, area: str):
        return self._areaEmail.update_area_email_area_by_name(name, area)

    def delete_collect_data_before_date(self, date: str):
        return self._collectData.delete_before_date(date)

    def delete_process_data_before_date(self, date: str):
        return self._processData.delete_before_date(date)

    def delete_process_result_data_before_date(self, date: str):
        return self._processResultData.delete_before_date(date)
